#!/usr/bin/env python

import os
import subprocess
import csv
import re
from pprint import pprint


def GetOptionalBool(entry, key, type):
	if entry[key]:
		if entry[key] == 'true' or entry[key] == 'True' or entry[key] == 1:
			return True
		elif entry[key] != 'false' and entry[key] != 'False' and entry[key] != 0:
			print "ERROR: Invalid " + key + " field for " + type
	return False

def WriteOptionalBool(defFile, entry, key, type):
	if (GetOptionalBool(entry, key, type)):
		defFile.write("info." + key + " = true;\n")

def WriteEnum(defFile, entry, key, dict, type):
	value = entry[key]
	if not value:
		print "ERROR: Missing " + key + " field for " + type
		return

	enum = dict[value]	
	if not enum:
		print "ERROR: Invalid " + key + " field for " + type
		return
	
	defFile.write("info." + key + " = " + enum + ";\n")

def WriteOptionalKey(defFile, entry, key, type):
	if entry[key]:
		defFile.write("info." + key + " = " + entry[key] + ";\n")

def WriteOptionalString(defFile, entry, key, type):
	if entry[key]:
		defFile.write("info." + key + ' = "' + entry[key] + '";\n')

def WriteOptionalTags(defFile, entry, key, type):
	if entry[key]:
		tags = entry[key].split(',')
		enums = ""
		for tag in tags:
			if (enums != ""):
				enums += " | " 
			enums += "OccupationTags::" + tag
		defFile.write("info." + key + ' = ' + enums + ';\n')

def WriteOptionalAccessories(defFile, entry, key, type):
	if entry[key]:
		accessories = entry[key].split(',')
		for accessory in accessories:
			defFile.write("info." + key + '[info.' + key + 'Count++] = AccessoryType::' + accessory + ';\n')


def WritePeriodFlags(defFile, entry, key, periods, type):
	periodFlags = entry[key]
	if not periodFlags:
		print "ERROR: No period for " + type
		return
	
	txt = ""
	for c in periodFlags:
		enum = periods[c]	
		if not enum:
			print "ERROR: Invalid period " + c + " for " + type
			return
		if txt != "":
			txt += " | "
		txt += enum

	defFile.write('info.periodFlags = ' + txt + ';\n')


def multipleReplace(text, wordDict):
	for key in wordDict:
		text = text.replace(key, wordDict[key])
		
	# Check that there are no unsubstituted strings
	found = re.search( r'\[\[(.*)\]\]', text)
	if found:
		print "ERROR: Unmatched replacement string in: " + text
	return text
    
def readVarsFrom(filename):
	vars = {}	
	ins = open(filename, "r" )
	array = []
	for line in ins:
		m = re.search("([a-zA-Z\d]+)\s*=\s*([\-0-9.]+)", line)
		if m:
			#print m.group(1) + " = " + m.group(2)
			#vars[m.group(1)] = float(m.group(2))
			key = '[[' + m.group(1) + ']]'
			vars[key] = m.group(2)
	ins.close()
	
	return vars

def readGlobalVars(dir):
	globalVars = {}
	globalVars.update(readVarsFrom(dir + "OccupationParams.h"))
	return globalVars

def main():
	dir = os.path.dirname(os.path.realpath(__file__))
	baseDir = dir + "/../LastingLegacy/AssetsRaw/"
	reader = csv.DictReader(open(baseDir + 'db/occupations.csv', 'rU'))

	sourceCodeDir = baseDir + "../src/"
	globalVariables = readGlobalVars(sourceCodeDir)
	pattern = re.compile(r'\b(' + '|'.join(globalVariables.keys()) + r')\b')

	Rarities = {'C': 'Rarity::Common' , 'U': 'Rarity::Uncommon', 'R': 'Rarity::Rare', 'Q': 'Rarity::Unique'}
	Periods = {'E': 'Period::Early' , 'M': 'Period::Middle', 'L': 'Period::Late'}

	defFile = open(sourceCodeDir + "Occupations.inc", 'w')
	for entry in reader:
		type = entry['id']
		defFile.write('{\nOccupationInfo& info = OccupationInfos[Occupation::' + type + '];\n')
		defFile.write('info.occupation = Occupation::' + type + ';\n')
		defFile.write('info.spriteId = IconSpriteId::' + type + ';\n')
		defFile.write('info.maleName = "' + entry['maleName'] + '";\n')
		if entry['femaleName']:
			defFile.write('info.femaleName = "' + entry['femaleName'] + '";\n')
		else:
			defFile.write('info.femaleName = info.maleName;\n')
		
		description = multipleReplace(entry['description'], globalVariables)
		defFile.write('info.description = "' + description +'";\n')
		
		defFile.write('info.logEntry = "' + entry['logEntry'] + '";\n')
		WriteOptionalBool(defFile, entry, 'educationRequired', type)
		WriteOptionalKey(defFile, entry, 'goldCost', type)
		WriteOptionalKey(defFile, entry, 'prestigeCost', type)
		WriteOptionalKey(defFile, entry, 'goldIncome', type)
		WriteOptionalKey(defFile, entry, 'prestigeIncome', type)
		WriteOptionalKey(defFile, entry, 'legacyIncome', type)
		WriteOptionalBool(defFile, entry, 'incomeMultiplied', type)
		WriteOptionalBool(defFile, entry, 'action', type)
		WriteEnum(defFile, entry, 'rarity', Rarities, type)
		WritePeriodFlags(defFile, entry, 'periods', Periods, type)
		WriteOptionalKey(defFile, entry, 'targetCount', type)
		WriteOptionalBool(defFile, entry, 'destructive', type)
		WriteOptionalBool(defFile, entry, 'pickable', type)
		WriteOptionalBool(defFile, entry, 'inherited', type)
		if entry['childrenOccupation']:
			defFile.write("info.childrenOccupation = Occupation::" + entry['childrenOccupation'] + ";\n")
		WriteOptionalKey(defFile, entry, 'personLike', type)
		WriteOptionalTags(defFile, entry, 'tags', type)
		WriteOptionalAccessories(defFile, entry, 'maleAccessories', type)
		WriteOptionalAccessories(defFile, entry, 'femaleAccessories', type)

		defFile.write('}\n')

	defFile.close()
	

if __name__ == '__main__':
    main()

